from typing import *
from typing import Any, TypeVar

T = TypeVar('T')
U = TypeVar('U')
BoolF = Callable[[bool, bool], bool]
GFunc = Tuple[Tuple[str, ...], Tuple[str, ...], str, Tuple[BoolF, ...], Tuple[BoolF, ...]]
GeneNm = str
geneNms: List[str]

class Vec3:
    x: float = ...
    y: float = ...
    z: float = ...
    def __init__(self, x: float, y: float, z: float) -> None: ...
    def dist(self, v1: Any) -> float: ...
    def toOrganism(self, sep: str=...) -> str: ...
    def swapZX(self) -> None: ...
    def toArray(self) -> None: ...
    def toTV(self) -> str: ...

class Cell:
    cid: int = ...
    pos: Vec3 = ...
    vol: float = ...
    exprs: Dict[GeneNm, bool] = ...
    def __init__(self, cid: int, pos: Vec3, vol: float, exprs: Dict[GeneNm, bool]=...) -> None: ...
    def dist(self, c1: Cell) -> float: ...
    geneNms: List[GeneNm] = ...
    def addExprs(self, exprs: Dict[str, bool], geneNms: List[str]) -> Any: ...
    def updateExprs(self, gfs: Dict[GeneNm, GFunc]) -> Any: ...
    def filterGs(self, e: Any) -> Any: ...
    def updateExprsC(self, c: Cell) -> Any: ...
    def swapZX(self) -> None: ...
    @classmethod
    def fromTV(cls: Any, nelems: Dict[str, str]) -> Cell: ...
    @classmethod
    def fromImage(cls: Any, cd: Any, res: List[float]) -> Cell: ...
    def toOrganism(self) -> str: ...
    def toText(self, sep: str=...) -> str: ...
    def centreToArray(self) -> None: ...
    def getOnGenes(self) -> None: ...
    def areExpressed(self, gs: List[str]) -> bool: ...
    def getGeneStatus(self, geneNm: str) -> int: ...
    def toTV(self) -> str: ...

class Tissue:
    cells: Dict[int, Cell] = ...
    neighs: Dict[int, Dict[int, float]] = ...
    L1_cells: Set[int] = ...
    def __init__(self, cells: Dict[int, Cell], neighs: Dict[int, Dict[int, float]], ecellIds: Set[int]=...) -> None: ...
    def __iter__(self) -> Iterator[Cell]: ...
    @classmethod
    def fromTV(cls: Any, fnGeom: str, fnNeigh: str) -> Tissue: ...
    @classmethod
    def fromImage(cls: Any, data: Any, res: List[float]) -> Tissue: ...
    def adjView(self) -> None: ...
    def relabelFromMap(self, relabelM: Dict[int, int]) -> None: ...
    def relabel(self) -> None: ...
    def filterL1(self) -> None: ...
    def toNeigh(self) -> str: ...
    def toOrganism(self) -> str: ...
    def toTVGeom(self) -> str: ...
    def toTVNeigh(self) -> str: ...
    def toTV(self) -> Tuple[str, str]: ...
    def centresToArray(self) -> None: ...
    def getGeneStatus(geneNm: Any, self: Any) -> List[Tuple[int, int]]: ...

class STissue(Tissue):
    def __init__(self, cells: Dict[int, Cell], neighs: Dict[int, Dict[int, float]], gexprs: Dict[int, Dict[GeneNm, bool]]=..., geneNms: List[str]=...) -> None: ...
    geneNms: List[GeneNm] = ...
    def addExprs(self, gexprs: Dict[int, Dict[str, bool]], geneNms: List[str]) -> Any: ...
    def relabel(self) -> None: ...
    def toOrganism(self) -> str: ...
    def updateExprsBF(self, gfs: Dict[GeneNm, GFunc]) -> Any: ...
    def updateExprsTs(self, ts: Tissue, iLn: Dict[int, int]) -> Any: ...
    def toDict(self) -> Dict[int, Dict[str, int]]: ...
    def toCSV(self) -> str: ...
    def filterGBExpr(self, e: Any) -> Any: ...
    def filterGs(self, gs: List[str]) -> List[int]: ...
    def filterL1(self) -> None: ...

def readImage(fseg: str) -> Any: ...
def readImageRegionPoints(fseg: Any, res: Any=..., d: int=...) -> Any: ...
def mergePoints(cell_regions: Any, cids: Any) -> Any: ...
def getCellRegions(fid: int=..., ft: Any=..., d: int=...) -> Any: ...
def readExprs(fn: str) -> Dict[int, Dict[str, bool]]: ...
def writeOrg(fnGeom: Any, fnNeigh: Any, ts: Any) -> None: ...
def writeTV(ts: Any, d: Any, lab: Any) -> None: ...
def readRess(fn: str) -> Dict[int, List[float]]: ...
def mkOrgs(d: Any, dExprs: Any, dOut: Any) -> None: ...
def mkOrgGeomState(fim: Any, fseg: Any, fexpr: Any, fout: Any) -> None: ...
def mkOrgGeom(fim: Any, fseg: Any, fout: Any) -> None: ...
def ge(s: str) -> Any: ...
def evalOp(e: Any, env: Any) -> Any: ...
def evalB(e: Any, env: Any) -> Any: ...
def boolToInt(b: bool) -> int: ...
def boolsToInt(bs: List[bool]) -> List[int]: ...
def const2(a: T, b: U) -> U: ...
def folds1(vals: List[T], fs: Tuple[Callable[[U, T], U], ...], iVal: U) -> U: ...
def folds(vals: List[bool], fs: Tuple[BoolF, ...]) -> bool: ...
def evalI(intr: Any, vals: List[bool], fsAct: Tuple[BoolF, ...], fsRepr: Tuple[BoolF, ...]) -> bool: ...
def readDataT1(d: str, dExprs: str, t: int) -> STissue: ...
def readGeomT1(d: str, t: int) -> Tissue: ...
def readDataT1Im(d: str, dExprs: str, t: Any) -> Any: ...
def readGeomIm(d: Any, t: Any) -> Any: ...
def cellsToDat(fn: Any, cells: Any) -> None: ...
